import asyncio
import ssl
import time
from typing import Tuple

from mcsniperpy.util.logs_manager import Color as color
from mcsniperpy.util.logs_manager import Logger as log


class Account:
    def __init__(self, email, password, *sqs, **kwargs):
        if kwargs.get("acc_type") == "microsoft":
            log.error("MCsniperPY does not support Microsoft accounts at the moment.")
        self.email = email  # email
        self.password = password  # password
        self.sqs = sqs  # security question answers

        self.questions = []  # Actual questions
        self.answer_ids = []  # IDS of the security question answers

        self.username = ""  # Totally useless lol

        self.bearer = ""
        self.headers = {"Authorization": "Bearer TOKEN"}

        self.snipe_data = (
            "PUT /minecraft/profile/name/NAME HTTP/1.1\r\n"
            "Host: api.minecraftservices.com\r\n"
            "Authorization: Bearer TOKEN\r\n"
            "\r\n"
        ).encode()

        self.readers_writers = []

    def encode_snipe_data(self, name):
        self.snipe_data = (
            f"PUT /minecraft/profile/name/{name} HTTP/1.1\r\n"
            "Host: api.minecraftservices.com\r\n"
            f"Authorization: Bearer {self.bearer}\r\n"
            "\r\n"
        ).encode()

    def encode_snipe_data_proxy(self, name):
        self.snipe_data = (
            f"PUT https://api.minecraftservices.com/minecraft/profile/name/{name} HTTP/1.1\r\n"
            "Host: api.minecraftservices.com\r\n"
            f"Authorization: Bearer {self.bearer}\r\n"
            "\r\n"
        ).encode()

    async def authenticate(self, session) -> bool:
        resp, _, resp_json = await session.post(
            "https://authserver.mojang.com/authenticate",
            json={
                "agent": {"name": "Minecraft", "version": 1},
                "username": self.email,
                "password": self.password,
                "requestUser": "false",
            },
            headers={"Content-Type": "application/json"},
        )

        if resp.status == 200:
            self.headers[
                "Authorization"
            ] = f"Bearer {resp_json['accessToken']}"  # authorization header
            self.bearer = resp_json["accessToken"]
            return True
        return False

    async def get_questions(self, session) -> None:
        resp, _, resp_json = await session.get(
            "https://api.mojang.com/user/security/challenges", headers=self.headers
        )

        self.answer_ids = [question["answer"]["id"] for question in resp_json]
        self.questions = [question["question"]["question"] for question in resp_json]

        if resp.status == 200:
            return
        log.error(f"failed to get security questions for {self.email}")

    async def need_sqs(self, session) -> bool:
        resp, _, _ = await session.get(
            "https://api.mojang.com/user/security/location", headers=self.headers
        )

        return resp.status == 403

    async def submit_questions(self, session):

        if self.sqs == ():
            log.error(f"security questions for {self.email} not provided!")
            return False

        resp, _, _ = await session.post(
            "https://api.mojang.com/user/security/location",
            headers=self.headers,
            json=[
                {"id": self.answer_ids[0], "answer": self.sqs[0]},
                {"id": self.answer_ids[1], "answer": self.sqs[1]},
                {"id": self.answer_ids[2], "answer": self.sqs[2]},
            ],
        )

        # Yes, that's kinda ugly lol. whatever.

        if resp.status == 204:
            return True
        log.error(f"security questions for {self.email} are incorrect!")
        return False

    async def fully_authenticate(self, session):
        if not await self.authenticate(session):
            log.error(f"failed to auth {self.email}")
            return

        if not await self.need_sqs(session):
            log.info(
                f"{color.white}[{color.l_green}success{color.white}]"
                f"{color.reset} authenticated {self.email}"
            )
        else:
            await self.get_questions(session)
            if await self.submit_questions(session):
                log.info(
                    f"{color.white}[{color.l_green}success{color.white}]"
                    f"{color.reset} authenticated {self.email}"
                    " with security questions"
                )
            else:
                log.error(f"failed to authenticate {self.email}")

    async def snipe_connect(self) -> None:

        reader, writer = await asyncio.open_connection(
            "api.minecraftservices.com",
            443,
            ssl=ssl.SSLContext(),
            ssl_handshake_timeout=5,
        )
        log.debug(f"Connected on account {self.email}")
        writer.write(self.snipe_data[0:-2])

        await writer.drain()
        self.readers_writers.append((reader, writer))

    async def snipe(self, writer: asyncio.StreamWriter, do_log: bool = True) -> None:
        writer.write(self.snipe_data[-2:])
        await writer.drain()
        if do_log:
            log.info(f"sent request @ {time.time()}")

    async def snipe_read(
        self,
        name: str,
        reader: asyncio.StreamReader,
        writer: asyncio.StreamWriter,
        do_log: bool = True,
    ) -> Tuple[bool, str, float]:
        resp = await reader.read(12)
        now = time.time()
        status = int(resp[9:12])
        is_success = status < 300

        writer.close()
        await writer.wait_closed()

        if do_log:
            pretty_status = "%s%s%s" % (
                {False: color.l_red, True: color.l_green}.get(is_success, color.l_red),
                status,
                color.reset,
            )
            pretty_name = "%s%s%s" % (color.l_cyan, name, color.reset)

            log.info("[%s] [%s] @ %.10f" % (pretty_name, pretty_status, now))
        return is_success, self.email, now



import os
import sys
from typing import List

from mcsniperpy.util.classes.account import Account
from mcsniperpy.util.logs_manager import Logger as log
from mcsniperpy.util.request_manager import RequestManager


def is_float(num):
    """Checks if a number is a valid float

    Args:
        n: number to check
    Returns:
        a boolean saying if n is a valid float"""
    try:
        float(num)
        return True
    except ValueError:
        return False


def parse_accs(file_path) -> List[Account]:
    accounts = list()
    log.debug(f"accounts path: {file_path}")
    if os.path.isfile(file_path):
        lines = [line.strip().split(":") for line in open(file_path).readlines()]
    else:
        log.error("accounts.txt file not found!")
        close(1)
    # ^ reads every line from a file and splits into a :
    for (
        line
    ) in lines:  # This variable cannot be referenced before assignment due to close()

        if line[0].startswith("#"):
            pass  # This triggers is the line is a "comment"
        elif len(line) in (2, 5):
            accounts.append(
                Account(*line)
            )  # While this line is difficult to read, it creates an Account object.
        else:
            log.error(f"accounts.txt invalid account on line {lines.index(line) + 1}")

    log.debug("loaded accounts from file")

    if len(accounts) == 0:
        log.error(
            "No accounts were loaded from file. Please check accounts.txt and try again."
        )
        close(0)

    if len(accounts) == 1:
        log.info(f"{len(accounts)} account has been loaded from file.")
    else:
        log.info(f"{len(accounts)} accounts have been loaded from file.")
    return accounts


def parse_accs_string(accounts_string) -> List[Account]:
    accounts = list()
    lines = accounts_string.split("\n")
    for line in lines:
        line = line.split(":")

        if len(line) in (2, 5):
            accounts.append(Account(*line))
        else:
            log.error(f"accounts.txt invalid account on line {lines.index(line) + 1}")

    log.debug("loaded accounts from string")

    if len(accounts) == 0:
        log.error(
            "No accounts were loaded from string. Please check accounts.txt and try again."
        )
        close(0)

    return accounts


def find_acc_by_email(email, accounts):
    for acc in accounts:
        if acc.email == email:
            return acc
    return None


def close(code) -> None:
    sys.exit(code)


async def upcoming(
    session: RequestManager,
    length: int = 3,
    length_op: str = "",
    searches: int = 0,
    url: str = "https://api.kqzz.me/api/namemc/upcoming",
):
    full_url = f"{url}?length_op={length_op}&length={length}&searches={searches}"
    resp, _, resp_json = await session.get(full_url)
    if resp.status < 300:
        return resp_json

    return None



import pathlib

from setuptools import setup

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")


setup(
    name="MCsniperPY",
    version="0.20.0",
    description="Minecraft name sniper written in Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MCsniperPY/MCsniperPY",
    author="Kqzz",
    license="MIT",
    packages=["mcsniperpy", "mcsniperpy.util", "mcsniperpy.util.classes"],
    install_requires=["typer", "aiohttp", "colorama", "bs4"],
    entry_points={"console_scripts": ["mcsniperpy=mcsniperpy.cli:cli"]},
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",  # Again, pick a license
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



# pylint: disable=broad-except
import random

# pylint: disable=pointless-string-statement
"""All of this file is just to change ```py
async with session.method() as r:
```

to

```py
r, text, r_json = await request_manager.method()
```"""

USER_AGENTS = [
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:44.0) Gecko/20100101 Firefox/44.01",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/"
    "54.0.2840.71 Safari/537.36",
    "Mozilla/5.0 (Linux; Ubuntu 14.04) AppleWebKit/537.36 Chromium/35.0.1870.2 Safa"
    "ri/537.36",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41."
    "0.2228.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko"
    ") Chrome/42.0.2311.135 "
    "Safari/537.36 Edge/12.246",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/601.3.9 (KHTML, "
    "like Gecko) Version/9.0.2 Safari/601.3.9",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/47.0.2526.111 Safari/537.36",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:54.0) Gecko/20100101 Firefox/54.0",
]


# https://github.com/N0taN3rd/userAgentLists/blob/9b99634d90aac404422109f80a0f273ebc1907c5/useragents.py#L28


class RequestManager:
    def __init__(self, session):
        self.session = session
        # pylint: disable=invalid-name
        self.ua = random.choice(USER_AGENTS)

    async def get(self, *args, **kwargs):
        async with self.session.get(*args, **kwargs) as response:
            try:
                json_resp = await response.json()
            except Exception:
                json_resp = None
            return response, await response.text(), json_resp

    async def post(self, *args, **kwargs):
        async with self.session.post(*args, **kwargs) as response:
            try:
                json_resp = await response.json()
            except Exception:
                json_resp = None
            return response, await response.text(), json_resp

    async def put(self, *args, **kwargs):
        async with self.session.put(*args, **kwargs) as response:
            try:
                json_resp = await response.json()
            except Exception:
                json_resp = None
            return response, await response.text(), json_resp
